# ------------------------------------------------------------------------------
# Boolean Values

FALSE					= 0
TRUE					= 1

# ------------------------------------------------------------------------------
# Program Information

PI_NAME					= "DrakSync"
PI_WIZARD				= "WebSync Wizard"
PI_VERSION				= "9.0"
PI_LOCALE_NAME			= "draksync"
PI_LOCALE_DIR			= "/usr/share/locale"
PI_DRAKSYNC_ROOT_DIR	= "/usr/share/draksync"
PI_MODULES_DIR			= PI_DRAKSYNC_ROOT_DIR + "/modules"
PI_ICONS_DIR			= PI_DRAKSYNC_ROOT_DIR + "/icons"
PI_COMMANDS_DIR			= PI_DRAKSYNC_ROOT_DIR + "/commands"

# ------------------------------------------------------------------------------
# About this program

import gettext
gettext.bindtextdomain(PI_LOCALE_NAME, PI_LOCALE_DIR)
gettext.textdomain(PI_LOCALE_NAME)
_ = gettext.gettext

PI_ABOUT = _(PI_NAME) + _(" version ") + str(PI_VERSION) + "\n" + \
_("Copyright (C) 2000-2001 Mandrakesoft SA\n" \
  "RedHog (Egil Moeller) <redhog@mandrakesoft.com>\n" \
  "Gwenole Beauchesne <gbeauchesne@mandrakesoft.com>") + \
"\n\n" + \
_("""This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.""")

# ------------------------------------------------------------------------------
# File Types

FT_REGULAR_FILE			= 1
FT_DIRECTORY			= 2
FT_LINK					= 3

# ------------------------------------------------------------------------------
# Connection Types

CT_SSH					= 1		# Rsync - Secure Shell
CT_RSH					= 2		# Rsync - Remote Shell
CT_FTP					= 3		# File Transfer Protocol
CT_LOCAL				= 4		# Synchronize local directories

CT_STRINGS = {
	CT_SSH:				"CT_SSH",
	CT_RSH:				"CT_RSH",
	CT_FTP:				"CT_FTP",
	CT_LOCAL:			"CT_LOCAL"
	}

# ------------------------------------------------------------------------------
# Connection Status

CS_OFFLINE				= 0
CS_ONLINE				= 1

# ------------------------------------------------------------------------------
# Types of Synchronization

ST_LOCAL_IS_NEWER		= 1		# Local overwrites remote
ST_REMOTE_IS_NEWER		= 2		# Remote overwrites local
ST_EITHER_IS_NEWER		= 3		# Newer overwrites older

ST_STRINGS = {
	ST_LOCAL_IS_NEWER:	"ST_LOCAL_IS_NEWER",
	ST_REMOTE_IS_NEWER:	"ST_REMOTE_IS_NEWER",
	ST_EITHER_IS_NEWER:	"ST_EITHER_IS_NEWER"
	}

# ------------------------------------------------------------------------------
# Synchronization Commands
#
# There are two variants for insertion and removal:
# - individual files
# - operation on the whole directory in case we have a faster way to do that
#   (e.g. .tar.gz over FTP)

SC_LOCAL_MASK			= 0x10
SC_REMOTE_MASK			= 0x20
SC_COMMAND_MASK			= 0x0f

# Basic commands ID
SC_UPDATE				= 1
SC_INSERT				= 2
SC_REMOVE				= 3

def is_local_command(command):
	return ((command & SC_LOCAL_MASK) != 0)

def is_remote_command(command):
	return ((command & SC_REMOTE_MASK) != 0)

def command_id(command):
	return (command & SC_COMMAND_MASK)

SC_TRAVERSE_DIRECTORY	= 0								# Used only for traversing directories that require no update at all
SC_UPDATE_LOCAL			= SC_LOCAL_MASK | SC_UPDATE		# Local file exists but has to be updated to the remote file
SC_REMOVE_LOCAL			= SC_LOCAL_MASK | SC_REMOVE		# Local file no longer exists on remote and has to be removed from remote dir
SC_INSERT_LOCAL			= SC_LOCAL_MASK | SC_INSERT		# Local file didn't exist and will be created from the remote file
SC_UPDATE_REMOTE		= SC_REMOTE_MASK | SC_UPDATE	# Remote file exists and has to be updated to the local file
SC_REMOVE_REMOTE		= SC_REMOTE_MASK | SC_REMOVE	# Remote file no longer exists on local and has to be removed from remote dir
SC_INSERT_REMOTE		= SC_REMOTE_MASK | SC_INSERT	# Remote file didn't exist and will be created from the local one

SC_STRINGS				= {
	SC_TRAVERSE_DIRECTORY:	"Traverse Directory",
	SC_UPDATE_LOCAL:		"Update Local",
	SC_REMOVE_LOCAL:		"Remove Local",
	SC_INSERT_LOCAL:		"Add Local",
	SC_UPDATE_REMOTE:		"Update Remote",
	SC_REMOVE_REMOTE:		"Remove Remote",
	SC_INSERT_REMOTE:		"Add Remote"
	}

# ------------------------------------------------------------------------------
# Regular Expressions for FTP Directory Parsing

RE_LISTDIR_UNIX			= "([-ld])([-rwxs]{9})\s+(\d+)\s+(\w+)\s+(\w+)\s+(\d+)\s+"\
						  "(jan|feb|mar|apr|may|jun|jul|aug|sep|oct|nov|dec)\s+" \
						  "(\d?\d)\s+(\d\d\d\d|\d\d:\d\d)\s+(.+)"

# ------------------------------------------------------------------------------
# Name of the file containing time-stamps for FTP synchronization
# (the file is located in the user's local directory to synchronize)

FTP_TIME_STAMP_FILE		= ".draksync_stamp"

# ------------------------------------------------------------------------------
# Convert month value as string into its matching ID (January == 1)

MONTH_ID				= {
	"jan":  1, "feb":  2, "mar":  3, "apr":  4, "may":  5, "jun":  6,
	"jul":  7, "aug":  8, "sep":  9, "oct": 10, "nov": 11, "dec": 12
	}

# ------------------------------------------------------------------------------
# URLs to Mandrake websites

URL_MANDRAKE_EXPERT		= "http://www.mandrakeexpert.com/"
URL_MANDRAKE_QA			= "https://qa.mandrakesoft.com/"

# ------------------------------------------------------------------------------
# Menu items

MN_FILE					= _("/_File")
MN_FILE_WEBSYNC_WIZARD	= _("/File/_WebSync Wizard...")
MN_FILE_SEPARATOR		= _("/File/-")
MN_FILE_SYNC_ALL		= _("/File/S_ynchronize all")
MN_FILE_SAVE_AND_QUIT	= _("/File/_Save and quit")
MN_FILE_QUIT_WO_SAVING	= _("/File/_Quit without saving")
MN_GROUP				= _("/_Group")
MN_GROUP_SYNC			= _("/Group/S_ynchronize")
MN_GROUP_ADD			= _("/Group/_Add")
MN_GROUP_REMOVE			= _("/Group/_Remove")
MN_GROUP_RENAME			= _("/Group/Re_name")
MN_CONNECTION			= _("/_Connection")
MN_CONNECTION_SYNC		= _("/Connection/S_ynchronize")
MN_CONNECTION_ADD		= _("/Connection/_Add")
MN_CONNECTION_REMOVE	= _("/Connection/_Remove")
MN_CONNECTION_EDIT		= _("/Connection/_Edit")
MN_HELP					= _("/_Help")
MN_HELP_MANUAL			= _("/Help/_Manual")
MN_HELP_SEPARATOR		= _("/Help/-")
MN_HELP_MANDRAKE_EXPERT	= _("/Help/Mandrakeexpert.com")
MN_HELP_BUG_REPORT		= _("/Help/_Bug report...")
MN_HELP_ABOUT			= _("/Help/_About...")

# ------------------------------------------------------------------------------
# Popup menu items

MNP_GROUP_SYNC			= _("/Synchronize group")
MNP_GROUP_ADD			= _("/Add group")
MNP_GROUP_REMOVE		= _("/Remove group")
MNP_GROUP_RENAME		= _("/Rename group")
MNP_CONNECTION_SYNC		= _("/Synchronize connection")
MNP_CONNECTION_ADD		= _("/Add connection")
MNP_CONNECTION_REMOVE	= _("/Remove connection")
MNP_CONNECTION_EDIT		= _("/Edit connection")
MNP_ALL_SYNC			= _("/Synchronize all")
# no need to make this one translatable, it has nothing to be translated
MNP_SEPARATOR			=   "/-"

# ------------------------------------------------------------------------------
# Wizard buttons

WIZ_BUTTON_BACK			= "<- " + _("Back")
WIZ_BUTTON_NEXT			= _("Next") + " ->"
WIZ_BUTTON_CANCEL		= _("Cancel")
WIZ_BUTTON_FINISH		= _("Finish")

# ------------------------------------------------------------------------------
# NetMonitor related

NET_MONITOR_APP			= "/usr/sbin/net_monitor.real"
NET_MONITOR_APP_AUTH	= "/usr/sbin/net_monitor"

# ------------------------------------------------------------------------------
# Misc constants

BORDER_WIDTH = 5

# ------------------------------------------------------------------------------
# Local Variables:
# tab-width: 4
# End:
