import _gtk
from gtk import *
from GDK import *

from Constants import TRUE, FALSE						# Boolean Values
from Constants import PI_LOCALE_NAME, PI_LOCALE_DIR		# Locales stuff

# ------------------------------------------------------------------------------
# Localization

import gettext
gettext.bindtextdomain(PI_LOCALE_NAME, PI_LOCALE_DIR)
gettext.textdomain(PI_LOCALE_NAME)
_ = gettext.gettext

# Misc constants
GRAB_DEFAULT = TRUE

# ------------------------------------------------------------------------------
# GUI wrapper over Gtk

class GuiPopup:
	class ui_btn_pressed:
		def __init__(self, dialog, user):
			self.dialog = dialog
			self.userfn = user[0]
			self.userdata = user[1]
		def __call__(self, button):
			self.dialog.hide()
			if self.userfn:
				if self.userdata != None:
					apply(self.userfn, self.userdata)
				else:
					apply(self.userfn, ())

class GuiPopupQuestion(GtkDialog, GuiPopup):
	def __init__(self, title, question, cancel = (None, None), buttons = ()):
		GuiPopupQuestion.__bases__[0].__init__(self)
		_gtk.gtk_object_set_data(self._o, "userclass", self)
		self.set_position(WIN_POS_CENTER)
		self.set_title(title)
		self.set_modal(TRUE)
		self.connect("destroy", self.ui_btn_pressed(self, cancel))
		self.innervbox = GtkVBox(TRUE)
		self.innervbox.set_border_width(10)
		self.question = GtkLabel(question)
		self.question.set_line_wrap(TRUE)
		self.innervbox.pack_start(self.question, TRUE, TRUE, 0)
		self.vbox.add(self.innervbox)
		self.buttons = []
		self.default_button = None
		for button_info in buttons:
			# Tuple format: ( label , callback [, arg = None [, grab_default = FALSE ] ] )
			(label, callback), arg, grab_default  = button_info[0:2], None, FALSE
			if len(button_info) >= 3: arg = button_info[2]
			if len(button_info) >= 4: grab_default = button_info[3]
			button = GtkButton(label)
			self.action_area.pack_start(button)
			self.buttons.append(button)
			button.connect("clicked", self.ui_btn_pressed(self, (callback, arg)))
			# FIXME: set CAN_DEFAULT flag for *every* button since
			# non-"grab_default"'ed buttons will appear much bigger...
			button.set_flags(CAN_DEFAULT)
			if grab_default or len(self.buttons) == 1:
				button.grab_default()
				self.default_button = button
	def show(self):
		self.innervbox.show()
		self.question.show()
		for button in self.buttons:
			button.show()
		GuiPopupQuestion.__bases__[0].show(self)

class GuiPopupInput(GuiPopupQuestion):
	def __init__(self, title, question, orig, cancel = (None, None), buttons = ()):
		GuiPopupInput.__bases__[0].__init__(self, title, question, cancel, buttons)
		self.input = GtkEntry()
		self.input.set_text(orig)
		self.input.connect("activate", self.ui_enter_pressed)
		self.innervbox.pack_start(self.input, TRUE, TRUE, 0)
	def show(self):
		self.input.show()
		self.input.grab_focus()
		GuiPopupInput.__bases__[0].show(self)
	def ui_enter_pressed(self, *args):
		if self.default_button:
			self.default_button.clicked()
	class ui_btn_pressed(GuiPopup.ui_btn_pressed):
		def __call__(self, button):
			self.dialog.hide()
			if self.userfn:
				if self.userdata != None:
					apply(self.userfn, (self.dialog.input.get_text(),) + self.userdata)
				else:
					apply(self.userfn, (self.dialog.input.get_text(),))

class GuiPopupPasswordInput(GuiPopupInput):
	def __init__(self, title, question, orig, cancel = (None, None), buttons = ()):
		GuiPopupPasswordInput.__bases__[0].__init__(self, title, question, orig, cancel, buttons)
		self.input.set_visibility(FALSE)

class GuiPopupNewPasswordInput(GuiPopupPasswordInput):
	def __init__(self, title, question, orig, cancel = (None, None), buttons = ()):
		GuiPopupNewPasswordInput.__bases__[0].__init__(self, title, question, orig, cancel, buttons)
		self.input2 = GtkEntry()
		self.input2.set_text(orig)
		self.input2.set_visibility(FALSE)
		self.innervbox.pack_start(self.input2, TRUE, TRUE, 0)
	def show(self):
		self.input2.show()
		GuiPopupNewPasswordInput.__bases__[0].show(self)
	class ui_btn_pressed(GuiPopupPasswordInput.ui_btn_pressed):
		def __call__(self, button):
			self.dialog.hide()
			if self.userfn:
				if self.userdata != None:
					apply(self.userfn, (self.dialog.input.get_text(), self.dialog.input.get_text() == self.dialog.input2.get_text()) + self.userdata)
				else:
					apply(self.userfn, (self.dialog.input.get_text(), self.dialog.input.get_text() == self.dialog.input2.get_text()))

class GuiPopupNewPasswordInputOKCancel(GuiPopupNewPasswordInput):
	def __init__(self, title, question, orig, ok = (None, None), cancel = (None, None)):
		GuiPopupNewPasswordInputOKCancel.__bases__[0].__init__(self, title, question, orig, (self.ui_cancel, cancel[1]), ((_("OK"), self.ui_ok, ok[1]), (_("Cancel"), self.ui_cancel, cancel[1])))
		self.ok = ok[0]
		self.cancel = cancel[0]
	def ui_cancel(self, passwd, equal, *args):
		apply(self.cancel, args)
	def ui_ok(self, passwd, equal, *args):
		if equal:
			apply(self.ok, (passwd,) + args)
		else:
			GuiPopupQuestion(_(PI_NAME) + " - " + _("Passwords do not match"),
							 _("The two passwords you entered do not match.") + "\n" +
							 _("Please retry."),
							 (self.cancel, None), ((_("OK"), self.ui_retry, None, GRAB_DEFAULT),)).show()
	def ui_retry(self):
		self.show()

# This is just an ordinary file selection box, but with only directory
# selection widgets visible...
class GuiPopupDirectorySelection(GtkFileSelection, GuiPopup):
	def __init__(self, title, orig=None, cancel=(None, None), ok=(None, None)):
		GuiPopupDirectorySelection.__bases__[0].__init__(self, title)
		self.hide_fileop_buttons()
		scroller = _gtk.gtk_object_get(self.file_list._o, "parent")
		_gtk.gtk_widget_hide(scroller)
		if orig:
			self.set_filename(orig)
		self.connect("destroy", self.ui_btn_pressed(self, cancel))
		self.cancel_button.connect("clicked", self.ui_btn_pressed(self, cancel))
		self.ok_button.connect("clicked", self.ui_btn_pressed(self, ok))
	class ui_btn_pressed(GuiPopup.ui_btn_pressed):
		def __call__(self, button):
			self.dialog.hide()
			if self.userfn:
				if self.userdata != None:
					apply(self.userfn, (self.dialog.get_filename(),) + self.userdata)
				else:
					apply(self.userfn, (self.dialog.get_filename(),))

