DEBUG = 0

import os, string, time, pickle
import _gtk
from gtk import *
from Gui import *
from string import *
from popen2 import *

import Error
from Constants import TRUE, FALSE												# Boolean Values
from Constants import PI_NAME, PI_LOCALE_NAME, PI_VERSION, PI_ABOUT				# Program Information
from Constants import PI_COMMANDS_DIR, PI_LOCALE_DIR, PI_ICONS_DIR				# Program Directories
from Constants import ST_LOCAL_IS_NEWER, ST_REMOTE_IS_NEWER, ST_EITHER_IS_NEWER	# Synchronization types
from Constants import CT_RSH, CT_SSH, CT_FTP									# Protocols
from Constants import FTP_TIME_STAMP_FILE										# Time-stamp file name
from ftplib import FTP, FTP_PORT, error_reply, error_perm						# FTP library
from FileTree import FTPFileTree, LocalFileTree									# File Trees
from SyncFileTree import SyncFileTree, SyncFile									# Synchronized File Tree
from Connection import FTPConnection, LocalConnection							# Connection Types
from Constants import BORDER_WIDTH

# Unqueue pending GtkEvents
def gtk_iteration():
	while events_pending():
		mainiteration()

# ------------------------------------------------------------------------------
# Localization

import gettext
gettext.bindtextdomain(PI_LOCALE_NAME, PI_LOCALE_DIR)
gettext.textdomain(PI_LOCALE_NAME)
_ = gettext.gettext

# ------------------------------------------------------------------------------
# FTP Site Information

class FTPInfo:

	DEFAULT_HOSTNAME = "ftp.yourserver.com"
	DEFAULT_USERNAME = "my_login"
	DEFAULT_PASSWORD = None

	def __init__(self, hostname = None, username = None, password = None, local_dir = None, remote_dir = None):
		self.remember_password = FALSE
		
		if not hostname:
			hostname = FTPInfo.DEFAULT_HOSTNAME
		if not username:
			username = FTPInfo.DEFAULT_USERNAME
		if not password:
			password = FTPInfo.DEFAULT_PASSWORD
		if not remote_dir:
			remote_dir = "~/"
		if not local_dir:
			if os.environ.has_key("HOME"):
				local_dir = os.environ["HOME"] + "/"
			else:
				local_dir = ""
		
		self.hostname	= hostname
		self.username	= username
		self.password	= password
		self.local_dir	= local_dir
		self.remote_dir	= remote_dir

	def __str__(self):
		return	"Host      : " + self.hostname  + "\n" + \
				"Username  : " + self.username  + "\n" + \
				"Password  : " + self.password  + "\n" + \
				"Remember? : " + str(self.remember_password) + "\n" + \
				"Local dir : " + self.local_dir + "\n" + \
				"Remote dir: " + self.remote_dir

# ------------------------------------------------------------------------------
# Transfer (FTP protocol)

def FTPTransfer(sync_type, delete_files, ftpinfo, raise_on_failure = FALSE):

	class DoTransfer:
		def __init__(self, sync_type, delete_files, ftpinfo, raise_on_failure):
			self.protocol			= CT_FTP
			self.sync_type			= sync_type
			self.delete_files		= delete_files
			self.ftpinfo			= ftpinfo
			self.first_auth_attempt	= TRUE
			self.cancel				= FALSE
			self.done				= FALSE
			self.exception_caught	= None
			self.sync_file_tree		= None
			self.raise_on_failure	= raise_on_failure
			
			# (gb) Time (in seconds since Epoch, GMT) of last successful synchronization
			#
			# DESCRIPTION
			#
			# This datum is only used for FTP synchronization. Its purpose
			# is to tell DrakSync that at this time, we are sure that both
			# ends contain the same files. It acts as a "synchronization
			# barrier". The time-stamp file is stored in the user's local
			# directory to synchronize.
			#
			# FTP LIMITS
			#
			# We can't change the modtime on the remote server. Though
			# some servers support "SITE UTIME" for that purpose, we
			# simply can't use that since "some" tends to reduce to a
			# very little number of FTP servers. As of June, 2001, neither
			# wu-ftpd, nor proftd support that command.
			#
			# Why is that a problem then ?
			#
			# Suppose, we have file AL lying on the local directory but
			# the remote directory is currently empty. At T1, we make a
			# first synchronization in the local -> remote direction.  The
			# file doesn't exist on the remote server, it then copied.
			# Let's name it as AR.
			#
			# Now imagine that at T2, we want to do a new synchronization.
			# According to the problem above-mentioned, DrakSync will see
			# that AR is potentially more recent than AL. Their modtimes
			# differ, so AL is copied onto AR again, though AL has not
			# changed since T1.
			#
			# SYNCHRONIZATION BARRIER
			#
			# The idea of the "synchronization barrier" is to guarantee
			# that at some time, we know that both ends are synchronized.
			# In other words, they do contain the very same files but
			# their modtime may be different.
			#
			# Variable "last_sync" tells DrakSync when the last
			# _successful_ synchronization occured. Knowing that, we can
			# simply check which files changed from the last
			# synchronization and copy those file onto the remote server.
			#
			# CAVEATS
			#
			# This method works well if we are sure that there is only
			# one source of changes. In other words, we assume that the
			# remote directory will be altered _only_ by the _current_
			# local machine.
			#
			# Illustration: say we have a server S and two other machines
			# A and B. S have to be modified only by A or only by B.
			# Doing otherwise results in undefined behavior. In other
			# words, we don't know what happens because there is an
			# "untrusted window": [last_sync(A), current_synchronization]
			# and [last_sync(B), current_synchronization].
			#
			# Besides, even with one source of changes, the working
			# directory shall not be modified while performing
			# synchronization. That's obvious.
	
			self.remote_key = self.ftpinfo.username + "@" + self.ftpinfo.hostname + ":" + os.path.normpath(self.ftpinfo.remote_dir)
			self.last_sync = self.__get_time_stamp()
	
			# UI initialization
			self.main_window = GtkWindow()
			self.main_window.connect("delete_event", self.cancel_cb)
			self.main_window.set_default_size(400, 150)
			self.main_window.set_title(_("Synchronization in progress"))
			self.main_window.set_border_width(10)
	
			self.main_vbox = GtkVBox()
			self.label_remaining_files = GtkLabel(_("Please wait while determining the number of files to synchronize"))
			self.main_vbox.pack_start(self.label_remaining_files, TRUE, FALSE, 4)
			self.bar_all_files = GtkProgressBar()
			self.main_vbox.pack_start(self.bar_all_files, TRUE, FALSE, 4)
			self.label_current_file = GtkLabel(_("Current file: %s") % "")
			self.main_vbox.pack_start(self.label_current_file, TRUE, FALSE, 4)
			self.bar_current_file = GtkProgressBar()
			self.main_vbox.pack_start(self.bar_current_file, TRUE, FALSE, 4)
			self.cancel_button = GtkButton(_("Cancel"))
			self.cancel_button.connect("clicked", self.cancel_cb)
			self.main_vbox.pack_start(self.cancel_button, FALSE, FALSE, 4)
	
			self.main_window.add(self.main_vbox)

		def run(self):
			self.__initiate_connection()

			# Wait until transfer is done or failed
			while not self.done:
				gtk_iteration()
			
			if not self.cancel:
				# synchronization was successful, update time stamps in local directory
				time_stamp = int(time.time())
				self.__set_time_stamp(time_stamp)

			# Reraise the error if the synchronization failed, if requested
			if self.cancel and self.raise_on_failure:
				raise self.exception_caught

			self.main_window.destroy()
	
		def __get_time_stamps(self):
			# Try to load the time-stamps
			try:
				file = open(os.path.join(self.ftpinfo.local_dir, FTP_TIME_STAMP_FILE), "r")
			except IOError:
				time_stamps = { self.remote_key: 0 }
			else:
				time_stamps = pickle.load(file)
				file.close()
			return time_stamps
		
		def __get_time_stamp(self):
			time_stamps = self.__get_time_stamps()
			time_stamp = 0
			if time_stamps.has_key(self.remote_key):
				time_stamp = time_stamps[self.remote_key]
			return time_stamp
	
		def __set_time_stamp(self, time_stamp):
			time_stamps = self.__get_time_stamps()
			time_stamps[self.remote_key] = time_stamp
			file = open(os.path.join(self.ftpinfo.local_dir, FTP_TIME_STAMP_FILE), "w")
			pickle.dump(time_stamps, file)
			file.close()
											  
		def __initiate_connection(self):
			if self.first_auth_attempt:
				self.first_auth_attempt = FALSE
				if not self.ftpinfo.password:
					GuiPopupPasswordInput(_("%s - Password required for remote host") % PI_NAME,
										  _("The remote host requires a password for %s@%s.") % (self.ftpinfo.username, self.ftpinfo.hostname) + "\n\n" +
										  _("Please enter your password:"), "",
										  (None, None),
										  ((_("OK"), self.__password_ok_cb, None, GRAB_DEFAULT),
										   (_("Cancel"), self.__password_cancel_cb, None))).show()
				else:
					self.__password_ok_cb(self.ftpinfo.password)
			else:
				GuiPopupPasswordInput(_("%s - Permission denied") % PI_NAME,
									  _("Permission denied.") + " " +
									  _("Please enter your password again for %s@%s:") % (self.ftpinfo.username, self.ftpinfo.hostname), "",
									  (None, None),
									  ((_("OK"), self.__password_ok_cb, None, GRAB_DEFAULT),
									   (_("Cancel"), self.__password_cancel_cb, None))).show()
	
		def __password_ok_cb(self, text):
			self.ftpinfo.password = text
			try:
				self.__initiate_file_tree()
			except Error.LoginFailed:
				# Login failed, try again
				self.__initiate_connection()

		def __password_cancel_cb(self, text):
			self.cancel = TRUE
			self.exception_caught = Error.LoginFailed()
			self.done = TRUE

		def __ftp_create_directory(self, ftp, path):
			# Recursively create the requested directories
			# Returns TRUE on success, FALSE otherwise
			directory_created = TRUE
			orig_path = ftp.pwd()
			for dir in string.split(os.path.normpath(path), "/"):
				print dir
				try:
					ftp.cwd(dir)
				except error_perm, error_msg:
					if (str(error_msg)[:3] == "550"):
						try:
							ftp.mkd(dir)
						except error_perm:
							directory_created = FALSE
						else:
							ftp.cwd(dir)
			ftp.cwd(orig_path)
			return directory_created

		def __cb_ok_create_directory(self, ftp, path):
			#for x in string.split(os.path.normpath(path), "/"):
			print string.split(path, "/")
			#if self.__ftp_create_directory(ftp, path):
			#	self.exception_caught = None
			self.may_create_directory = FALSE

		def __cb_cancel_create_directory(self):
			self.may_create_directory = FALSE
			self.done = TRUE

		def __initiate_file_tree(self):
			try:
				remote_connection = FTPConnection(self.ftpinfo.hostname, self.ftpinfo.username, self.ftpinfo.password)
				remote_file_tree = FTPFileTree(remote_connection)
				remote_file_tree.getTree(self.ftpinfo.remote_dir)
			except Error.FTPError, error_msg:
				self.exception_caught = Error.FTPError(error_msg)
				GuiPopupQuestion(PI_NAME + " - " + _("FTP connection problem"), str(error_msg),
								 (None, None), ((_("OK"), self.error_cb, None, GRAB_DEFAULT),)).show()
			except Error.UnknownHost, dirname:
				self.exception_caught = Error.UnknownHost(dirname)
				GuiPopupQuestion(PI_NAME + " - " + _("Unknown host name"),
								 _("Remote host (%s) was not found. Please check the host name is correctly spelled and make sure you are online.") % self.ftpinfo.hostname,
								 (None, None), ((_("OK"), self.error_cb, None, GRAB_DEFAULT),)).show()
			except Error.ConnectionClosed:
				self.exception_caught = Error.ConnectionClosed()
				GuiPopupQuestion(_("%s - Connection closed") % PI_NAME,
								 _("Service not available, remote server closed the connection"),
								 (None, None), ((_("OK"), self.error_cb, None, GRAB_DEFAULT),)).show()
			except Error.DirectoryNotFound, dirname:
				if TRUE:
					self.exception_caught = Error.DirectoryNotFound(dirname)
					# TODO: popup dialog to ask for the creation of the remote directory/directories
					action = _("Please create the %s directory before attempting to synchronize your files.") % _("remote")
					GuiPopupQuestion(_("%s - Directory not found") % PI_NAME,
									 _("The specified %s directory (%s) does not exist.") % (_("remote"), dirname) + "\n\n" + action,
									 (None, None), ((_("OK"), self.error_cb, None, GRAB_DEFAULT),)).show()
				else:
					# FIXME: python fails with this code
					self.exception_caught = Error.DirectoryNotFound(dirname)
					if (self.sync_type == ST_REMOTE_IS_NEWER):
						GuiPopupQuestion(_("%s - Directory not found") % PI_NAME,
										 _("The specified %s directory (%s) does not exist.") % (_("remote"), dirname) + "\n\n",
										 _("Please check the exact directory spelling."),
										 (None, None), ((_("OK"), self.error_cb, None),)).show()
					else:
						# Does the user want to create the remote directory ?
						self.may_create_directory = TRUE
						GuiPopupQuestion(_("%s - Directory not found") % PI_NAME,
										 _("The specified %s directory (%s) does not exist.") % (_("remote"), dirname) + "\n\n" +
										 _("Should I recursively create it ?"),
										 (None, None),
										 ((_("OK"), self.__cb_ok_create_directory, (remote_connection, dirname)),
										  (_("Cancel"), self.__cb_cancel_create_directory, None, GRAB_DEFAULT))).show()
						
						while self.may_create_directory:
							gtk_iteration()
	
			if not self.exception_caught:
				try:
					local_connection = LocalConnection()
					local_file_tree = LocalFileTree(local_connection)
					local_file_tree.getTree(self.ftpinfo.local_dir)
				except Error.DirectoryNotFound, dirname:
					self.exception_caught = Error.DirectoryNotFound(dirname)
					# TODO: popup dialog to ask for the creation of the local directory/directories
					action = _("Please create the %s directory before attempting to synchronize your files.") % _("local")
					GuiPopupQuestion(_("%s - Directory not found") % PI_NAME,
									 _("The specified %s directory (%s) does not exist.") % (_("local"), dirname) + "\n\n" + action,
									 (None, None), ((_("OK"), self.error_cb, None, GRAB_DEFAULT),)).show()
			
			if not self.exception_caught:
				try:
					self.sync_file_tree = SyncFileTree(self.sync_type, local_file_tree, remote_file_tree)
					self.sync_file_tree.prepare(self.last_sync)
					self.__synchronize()
				except Error.ConnectionClosed:
					self.exception_caught = Error.ConnectionClosed()
					GuiPopupQuestion(_("%s - Connection closed") % PI_NAME,
									 _("Service not available, remote server closed the connection"),
									 (None, None), ((_("OK"), self.error_cb, None, GRAB_DEFAULT),)).show()
				except Error.FTPError, error_msg:
					self.exception_caught = Error.FTPError(error_msg)
					GuiPopupQuestion(PI_NAME + " - " + _("FTP connection problem"), str(error_msg),
									 (None, None), ((_("OK"), self.error_cb, None, GRAB_DEFAULT),)).show()
	
			# An error occured or transfer was completed: destroy dialog box in either case
			if self.exception_caught:
				self.cancel = TRUE
				while not self.done:
					gtk_iteration()
	
		def __gui_update_dir_entries(self, local_dir, remote_dir, dirname):
			# TODO: show current local and remote directories
			gtk_iteration()
			
		def __gui_update_file_entry(self, filename):
			if DEBUG:
				print filename
			self.label_current_file.set_text(_("Current file: %s") % filename)
			gtk_iteration()
			return self.cancel
	
		def __gui_update_file_progress(self, percent):
			self.bar_current_file.update(percent)
			gtk_iteration()
	
		def __gui_update_files_progress(self):
			self.file_count = self.file_count + 1
			self.bar_current_file.set_value(0)
			self.bar_all_files.update(float(self.file_count) / float(self.total_file_count))
			self.label_remaining_files.set_text(_("Files remaining to process: %d") % (self.total_file_count - self.file_count))
			gtk_iteration()
	
		def __count_files_cb(self, path, sync_file, __unused_arg):
			if sync_file.isfile():
				self.total_file_count = self.total_file_count + 1
	
		def __synchronize(self):
			self.main_window.show_all()
			
			self.file_count = 0
			self.total_file_count = 0
			self.sync_file_tree.traverse(self.__count_files_cb, None)
			self.label_remaining_files.set_text(_("Files remaining to process: %d") % self.total_file_count)
	
			self.bar_all_files.set_bar_style(PROGRESS_CONTINUOUS)
			self.bar_all_files.set_orientation(PROGRESS_LEFT_TO_RIGHT)
			self.bar_all_files.set_value(0)
			
			self.bar_current_file.set_bar_style(PROGRESS_CONTINUOUS)
			self.bar_current_file.set_orientation(PROGRESS_LEFT_TO_RIGHT)
			self.bar_current_file.set_value(0)
			
			self.sync_file_tree.synchronize((
				self.__gui_update_dir_entries,
				self.__gui_update_file_entry,
				self.__gui_update_file_progress,
				self.__gui_update_files_progress,
				self.delete_files, self.last_sync
				))

			self.cancel = FALSE
			self.done = TRUE
		
		def cancel_cb(self, *args):
			self.cancel = TRUE
			self.done = TRUE
	
		def error_cb(self, *args):
			self.done = TRUE

	transfer = DoTransfer(sync_type, delete_files, ftpinfo, raise_on_failure)
	transfer.run()


# ------------------------------------------------------------------------------
# Transfer (rsync protocol)

class Transfer:
	def __got_input__(self, *args):
		input = self.syncer.read(1)
		if input == "":
			self.__delete__()
		elif input == "\n":
			# These are the input categories:
			#ERROR[: .*]
			if find(self.input, "ERROR") == 0:
				_gtk.gtk_input_remove(self.input_tag)
				self.syncer.close()
				if self.input == "ERROR: PERMISSION DENIED":
					if self.first_time:
						GuiPopupPasswordInput(PI_NAME + " - " + _("Passphrase required for host"),
											  _("The remote host requires a passphrase for %s@%s.") % (self.user, self.host) + "\n\n" +
											  _("Please enter your passphrase:"),
											  "", (self.__password_cancel__, None),
											  ((_("OK"), self.__password_ok__, None, GRAB_DEFAULT), (_("Cancel"), self.__password_cancel__, None))).show()
					else:
						GuiPopupPasswordInput(PI_NAME + " - " + _("Permission denied"),
											  _("Permission denied. Please reenter passphrase for %s@%s:") % (self.user, self.host),
											  "", (self.__password_cancel__, None),
											  ((_("OK"), self.__password_ok__, None, GRAB_DEFAULT), (_("Cancel"), self.__password_cancel__, None))).show()
					self.first_time = FALSE
				elif self.input == "ERROR: NO SSH-AGENT":
					identity.add((self.__initiate_sync__, None), ( self.__delete__, None))
				elif self.input == "ERROR: CONNECTION CLOSED":
					GuiPopupQuestion(PI_NAME + " - " + _("Remote host closed the connection"),
									 _("The remote host closed the connection for %s@%s.") % (self.user, self.host) + "\n" +
									 _("This is probably due to that either the destination or the source directory (or both) does not exist.") + "\n" +
									 _("Please create the directory/ies before attempting to synchronize them."),
									 (None, None), ((_("OK"), None, None, GRAB_DEFAULT),)).show()
					self.__delete__()
				else:
					GuiPopupQuestion(PI_NAME + " - " + _("Unknown error"),
									 _("An unknown error '%s' has occurred.") % self.input,
									 (None, None), ((_("OK"), None, None, GRAB_DEFAULT),)).show()
					self.__delete__()
			#NUMBER OF FILES TO TRANSFER [IN (ONE|THE OTHER DIRECTION)]
			elif find(self.input, "NUMBER OF FILES TO TRANSFER") == 0:
				if find(self.input, "NUMBER OF FILES TO TRANSFER:") == 0 or find(self.input, "NUMBER OF FILES TO TRANSFER IN ONE DIRECTION:") == 0:
					self.number_of_files_to_transfer_in_one_direction = int(self.input[find(self.input, ":") + 1:])
				else:
					self.number_of_files_to_transfer_in_the_other_direction = int(self.input[find(self.input, ":") + 1:])
			#TRANSFERING [IN (ONE|THE OTHER) DIRECTION]
			elif find(self.input, "TRANSFERING") == 0:
				if self.input == "TRANSFERING" or self.input == "TRANSFERING IN ONE DIRECTION":
					self.eachfile = 1
					self.all.configure(0, 0, self.number_of_files_to_transfer_in_one_direction);
					if self.number_of_files_to_transfer_in_one_direction > 1:
						self.all.set_bar_style(PROGRESS_DISCRETE)
						self.all.set_discrete_blocks(self.number_of_files_to_transfer_in_one_direction)
					else:
						self.all.set_bar_style(PROGRESS_CONTINUOUS)
					self.all.set_orientation(PROGRESS_LEFT_TO_RIGHT)
				elif self.input == "TRANSFERING IN THE OTHER DIRECTION":
					self.eachfile = -1
					self.all.configure(self.number_of_files_to_transfer_in_the_other_direction, 0, self.number_of_files_to_transfer_in_the_other_direction);
					if self.number_of_files_to_transfer_in_the_other_direction > 1:
						self.all.set_bar_style(PROGRESS_DISCRETE)
						self.all.set_discrete_blocks(self.number_of_files_to_transfer_in_the_other_direction)
					else:
						self.all.set_bar_style(PROGRESS_CONTINUOUS)
					self.all.set_orientation(PROGRESS_RIGHT_TO_LEFT)
			#TRANSFERED (BYTES|PERCENT): [0123456789]*
			elif find(self.input, "TRANSFERED") == 0:
				if find(self.input, "TRANSFERED PERCENT") == 0:
					self.current.set_value(int(self.input[find(self.input, ":") + 1:]))
			#TRANSFER [IN (ONE|THE OTHER) DIRECTION] DONE
			elif find(self.input, "TRANSFER ") == 0:
				if find(self.input, "TRANSFER DONE") == 0 or find(self.input, "TRANSFER IN THE OTHER DIRECTION DONE") == 0:
					self.__delete__()
					return
			#ONE MORE FILE TRANSFERED
			elif self.input == "ONE MORE FILE TRANSFERED":
				self.all.set_value(self.all.get_value() + self.eachfile)
				self.current.set_value(0)
			self.input = ""
		else:
			self.input = self.input + input
	def __initiate_sync__(self):
		if self.protocol == CT_SSH or self.protocol == CT_RSH:
			local = os.path.expanduser(self.local)
			if local[-1:] != '/':
				local = local + "/"
			if self.host != "localhost" or self.user != os.environ["USER"]:
				remote = self.user + "@" + self.host + ":" + self.remote
			else:
				remote = os.path.expanduser(self.remote)
			if remote[-1:] != '/':
				remote = remote + "/"
			if self.protocol == CT_SSH:
				os.environ["RSYNC_RSH"] = 'ssh'
			else:
				os.environ["RSYNC_RSH"] = 'rsh'
			if self.delete_files:
				os.environ["DELETE_FILES"] = 'yes'
			else:
				os.environ["DELETE_FILES"] = 'no'
			if self.newer == ST_EITHER_IS_NEWER:
				command = PI_COMMANDS_DIR + "/none_is_newer.exp"
			else:
				command = PI_COMMANDS_DIR + "/some_is_newer.exp"
			if self.newer == ST_LOCAL_IS_NEWER:
				dirs = (local, remote)
			else:
				dirs = (remote, local)
			self.syncer, self.syncerin = popen2(PI_COMMANDS_DIR + "/unsetlang " + command + " \"" + dirs[0] + "\" \"" + dirs[1] + "\" 2>&1", 0)
			self.syncerin.write("PASSWORD: " + self.passwd + "\n")
			self.input_tag = _gtk.gtk_input_add(self.syncer.fileno(), INPUT_READ, self.__got_input__);
		elif self.protocol == CT_FTP:
			pass
		else:
			GuiPopupQuestion(PI_NAME + " - " + _("Bug: ") + _("Unsupported protocol"),
							 _("The protocol specified is not supported.") + " " + _("This is a bug - please report this."),
							 (None, None), ((_("OK"), None, None, GRAB_DEFAULT),)).show()
			self.__delete__()
	def __password_ok__(self, text):
		self.passwd = text
		self.__initiate_sync__()  
	def __password_cancel__(self, text):
		self.__delete__()
	def __init__(self, protocol=CT_SSH, local="~/tmp", user="guest", host="localhost", remote="~/tmp", newer=ST_EITHER_IS_NEWER, delete_files=TRUE):
		self.protocol = protocol
		self.local = local
		self.user = user
		self.host = host
		self.remote = remote
		self.newer = newer
		self.delete_files = delete_files
		self.passwd = ""
		self.first_time = 1
		self.input = ""
		self.main = GtkWindow()
		self.main.connect("delete_event", self.ui_delete)
		self.main.set_default_size(400, 150)
		self.main.set_title(_("Synchronization in progress"))
		self.main.set_border_width(10)
		self.container = GtkVBox()
		if newer == ST_REMOTE_IS_NEWER:
			self.title = GtkLabel(_("Synchronizing %s with %s@%s:%s.") % (self.local, self.user, self.host, self.remote) + " " + _("Please wait."))
		elif newer == ST_LOCAL_IS_NEWER:
			self.title = GtkLabel(_("Synchronizing %s@%s:%s with %s.") % (self.user, self.host, self.remote, self.local) + " " + _("Please wait."))
		else:
			self.title = GtkLabel(_("Synchronizing %s and %s@%s:%s.") % (self.local, self.user, self.host, self.remote) + " " + _("Please wait."))
		self.container.pack_start(self.title, 1, 0, 0)
		self.all_title = GtkLabel(_("Files:"))
		self.container.pack_start(self.all_title, 1, 0, 0)
		self.all = GtkProgressBar()
		self.all.set_bar_style(PROGRESS_DISCRETE)
		self.container.pack_start(self.all, 1, 0, 0)
		self.current_title = GtkLabel(_("Current file:"))
		self.container.pack_start(self.current_title, 1, 0, 0)
		self.current = GtkProgressBar()
		self.current.set_bar_style(PROGRESS_CONTINUOUS)
		self.current.set_orientation(PROGRESS_LEFT_TO_RIGHT)
		self.container.pack_start(self.current, 1, 0, 0)
		self.cancel = GtkButton(_("Cancel"))
		self.cancel.connect("clicked", self.ui_delete)
		self.container.pack_start(self.cancel, 1, 0, 0)
		self.main.add(self.container)
		self.main.show_all()
		self.__initiate_sync__()
	def __delete__(self):
		if not self.syncer.closed:
			_gtk.gtk_input_remove(self.input_tag)
			self.syncer.close()
		self.current.destroy()
		self.current_title.destroy()
		self.all.destroy()
		self.all_title.destroy()
		self.title.destroy()
		self.container.destroy()
		self.main.destroy()
		del self
	def ui_delete(self, *args):
		self.__delete__()
		return TRUE

# ------------------------------------------------------------------------------
# Create SSH identity (RSA1 -- ssh v1)

class CreateIdentity:
	def __got_input__(self, *args):
		input = self.creation.read(1)
		if input == "":
			if not self.creation.closed:
				_gtk.gtk_input_remove(self.input_tag)
				self.creation.close()
			if self.cancel[0]:
				if self.cancel[1]:
					apply(self.cancel[0], self.cancel[1])
				else:
					apply(self.cancel[0], ())
			del self
		elif input == "\n":
			# These are the input categories:
			#WARNING[: .*]
			if find(self.input, "WARNING") == 0:
				if self.input == "WARNING: IDENTITY ALLREADY CREATED":
					GuiPopupQuestion(PI_NAME + " - " + _("Ssh identity already created"),
									 _("You already have an ssh identity."),
									 (None, None), ((_("OK"), None, None, GRAB_DEFAULT),)).show()
				else:
					GuiPopupQuestion(PI_NAME + " - " + _("Unknown warning"),
									 _("An unknown warning '%s' was issued.") % self.input,
									 (None, None), ((_("OK"), None, None, GRAB_DEFAULT),)).show()
			elif self.input == "DONE":
				if not self.creation.closed:
					_gtk.gtk_input_remove(self.input_tag)
					self.creation.close()
				if self.cont[0]:
					if self.cont[1]:
						apply(self.cont[0], self.cont[1])
					else:
						apply(self.cont[0], ())
				del self
				return
			self.input = ""
		else:
			self.input = self.input + input
	def __init__(self, passwd="", cont=(None, None), cancel=(None, None)):
		self.cont = cont
		self.cancel = cancel
		self.passwd = passwd
		self.input = ""
		self.creation, self.creationin = popen2(PI_COMMANDS_DIR + "/unsetlang " + PI_COMMANDS_DIR + "/create_identity.exp 2>&1", 0)
		self.creationin.write("PASSWORD: " + self.passwd + "\n")
		self.input_tag = _gtk.gtk_input_add(self.creation.fileno(), INPUT_READ, self.__got_input__);

# ------------------------------------------------------------------------------
# Transfer SSH identity

class TransferIdentity:
	def __got_input__(self, *args):
		input = self.transfer.read(1)
		if input == "":
			self.__delete__()
		elif input == "\n":
			# These are the input categories:
			#ERROR[: .*]
			if find(self.input, "ERROR") == 0:
				_gtk.gtk_input_remove(self.input_tag)
				self.transfer.close()
				if self.input == "ERROR: PERMISSION DENIED":
					GuiPopupPasswordInput(PI_NAME + " - " + _("Permission denied"),
										  _("Permission denied. Please reenter passphrase for %s@%s:") % (self.user, self.host),
										  "", (self.__password_cancel__, None),
										  ((_("OK"), self.__password_ok__, None, GRAB_DEFAULT), (_("Cancel"), self.__password_cancel__, None))).show()
				elif self.input == "ERROR: NO IDENTITY TO TRANSFER":
					GuiPopupNewPasswordInputOKCancel(PI_NAME + " - " + _("Create ssh identity"),
													 _("You don't have any ssh identity.") + "\n" +
													 _("If you want to create one, enter a passphrase for it and press OK:"),
													 "", (self.__create_identity__, None, GRAB_DEFAULT), (self.__create_identity_cancel__, None)).show()
				elif self.input == "ERROR: CONNECTION CLOSED":
					GuiPopupQuestion(PI_NAME + " - " + _("Connection closed"),
									 _("The connection was closed by %s@%s. Please ensure that sshd is running on the remote host.") % (self.user, self.host),
									 (None, None), ((_("OK"), None, None, GRAB_DEFAULT),)).show()
					self.__delete__()
				else:
					GuiPopupQuestion(PI_NAME + " - " + _("Unknown error"),
									 _("An unknown error '%s' has occurred.") % self.input,
									 (None, None), ((_("OK"), None, None, GRAB_DEFAULT),)).show()
					self.__delete__()
			#WARNING[: .*]
			elif find(self.input, "WARNING") == 0:
				if self.input == "WARNING: IDENTITY ALLREADY UPLOADED":
					GuiPopupQuestion(PI_NAME + " - " + _("Ssh identity already installed"),
									 _("Your ssh identity has already been uploaded to %s@%s.") % (self.user, self.host),
									 (None, None), ((_("OK"), None, None, GRAB_DEFAULT),)).show()
				else:
					GuiPopupQuestion(PI_NAME + " - " + _("Unknown warning"),
									 _("An unknown warning '%s' was issued.") % self.input,
									 (None, None), ((_("OK"), None, None, TRUE),)).show()
			elif self.input == "TRANSFERING IDENTITY":
				self.label.set_text(_("Transferring identity to %s@%s...") % (self.user, self.host))
			elif self.input == "INSTALLING IDENTITY":
				self.label.set_text(_("Installing identity on %s@%s...") % (self.user, self.host))
			elif self.input == "DONE":
				self.__delete__()
			self.input = ""
		else:
			self.input = self.input + input
	def __initiate_transfer__(self):
		self.transfer, self.transferin = popen2(PI_COMMANDS_DIR + "/unsetlang " + PI_COMMANDS_DIR + "/upload_identity.exp \"" + self.user + "\" \"" + self.host + "\" 2>&1", 0)
		self.transferin.write("PASSWORD: " + self.passwd + "\n")
		self.input_tag = _gtk.gtk_input_add(self.transfer.fileno(), INPUT_READ, self.__got_input__);
	def __password_ok__(self, text):
		self.passwd = text
		self.__initiate_transfer__()  
	def __password_cancel__(self, text):
		self.__delete__()
	def __create_identity_cancel__(self):
		self.__delete__()
	def __create_identity__(self, passwd):
		CreateIdentity(passwd, (self.__identity_created__, None), (self.__identity_creation_error__, None))
	def __identity_created__(self):
		self.__initiate_transfer__()
	def __identity_creation_error__(self):
		GuiPopupQuestion(PI_NAME + " - " + _("Unable to create ssh identity"),
						 _("An error occurred while creating your ssh identity. This may be a bug, or a) you don't have permission to create files in your own homedir, or b) you don't have ssh properly installed."), (None, None), ((_("OK"), None, None, GRAB_DEFAULT),)).show()
		self.__delete__()
	def __init__(self, passwd="", user="", host=""):
		self.passwd = passwd
		self.user = user
		self.host = host
		self.input = ""
		self.main = GtkWindow()
		self.main.set_title(PI_NAME + " - " + _("Uploading ssh identity"))
		self.main.set_border_width(BORDER_WIDTH)
		self.label = GtkLabel(_("Connecting to %s@%s...") % (self.user, self.host))
		self.main.add(self.label)
		self.main.show_all()
		self.__initiate_transfer__()
	def __delete__(self):
		if not self.transfer.closed:
			_gtk.gtk_input_remove(self.input_tag)
			self.transfer.close()
		self.label.destroy()
		self.main.destroy()
		del self
	def ui_delete(self, *args):
		self.__delete__()
		return TRUE

