DEBUG = 0

import _gtk, os
from gtk import *
from GDK import *
from Gui import *

import Error
from Constants import TRUE, FALSE																# Boolean Values
from Constants import PI_NAME, PI_WIZARD, PI_LOCALE_NAME, PI_VERSION, PI_ABOUT					# Program Information
from Constants import PI_COMMANDS_DIR, PI_LOCALE_DIR, PI_ICONS_DIR								# Program Directories
from Constants import ST_LOCAL_IS_NEWER, ST_REMOTE_IS_NEWER, ST_EITHER_IS_NEWER					# Synchronization types
from Constants import WIZ_BUTTON_BACK, WIZ_BUTTON_NEXT, WIZ_BUTTON_CANCEL, WIZ_BUTTON_FINISH	# Wizard Buttons
from Constants import NET_MONITOR_APP, NET_MONITOR_APP_AUTH										# Paths to Net Monitor
from Transfer import FTPInfo, FTPTransfer														# FTP Transfer
from Network import Network																		# Network Connection

# ------------------------------------------------------------------------------
# Localization

import gettext
gettext.bindtextdomain(PI_LOCALE_NAME, PI_LOCALE_DIR)
gettext.textdomain(PI_LOCALE_NAME)
_ = gettext.gettext

# ------------------------------------------------------------------------------
# Utility Functions

WIZARD_DEFAULT_PADDING = 4

# Make a table cols x rows and add it to the top widget
def make_table(top, cols, rows):
	table = GtkTable(cols, rows, FALSE)
	top.pack_start(table, FALSE, FALSE, WIZARD_DEFAULT_PADDING)
	return table

# Make an editable entry with the specified label and default value
# Possibly add an [...] extension button for file selector or whatever
def make_entry(top, label, value, extension = None):
	box = GtkHBox(FALSE, 4)
	top.pack_start(box, FALSE, FALSE, WIZARD_DEFAULT_PADDING)
	
	label = GtkLabel(label)
	box.pack_start(label, FALSE, FALSE, WIZARD_DEFAULT_PADDING)
		
	entry = GtkEntry()
	if value != None:
		entry.set_text(value)
	box.pack_start(entry, TRUE, TRUE, WIZARD_DEFAULT_PADDING);
	
	if extension != None:
		box.pack_start(extension, FALSE, FALSE, WIZARD_DEFAULT_PADDING)
		
	return entry

# Make a check box button
def make_checkbox(top, label, pressed, callback):
	button = GtkCheckButton(label)
	button.set_state(pressed)
	button.signal_connect("toggled", callback, button)
	top.pack_start(button, FALSE, FALSE, WIZARD_DEFAULT_PADDING)
	return button

# Add an entry (text, field) to a 2 x N table
def make_table_entry(table, row, label, value, extension = None):
	label = GtkLabel(label)
	label.set_justify(JUSTIFY_LEFT)
	
	table.attach(label, 0, 1, row, row + 1,
		     0, EXPAND | FILL,
		     WIZARD_DEFAULT_PADDING, WIZARD_DEFAULT_PADDING)
	
	entry = GtkEntry()
	if value != None:
		entry.set_text(value)
	
	box = GtkHBox(FALSE, 4)
	box.pack_start(entry, TRUE, TRUE, WIZARD_DEFAULT_PADDING);
	
	if extension != None:
		box.pack_start(extension, FALSE, FALSE, WIZARD_DEFAULT_PADDING)
	
	table.attach(box, 1, 2, row, row + 1,
		     EXPAND | FILL, EXPAND | FILL,
		     WIZARD_DEFAULT_PADDING, WIZARD_DEFAULT_PADDING)

	return entry

# Add a check box to a 2 x N table
def make_table_checkbox(table, row, label, pressed, callback):
	button = GtkCheckButton(label)
	button.set_state(pressed)
	button.signal_connect("toggled", callback, button)
	table.attach(button, 0, 2, row, row + 1,
		     EXPAND | FILL, EXPAND | FILL,
		     WIZARD_DEFAULT_PADDING, WIZARD_DEFAULT_PADDING)
	return button

# ------------------------------------------------------------------------------
# Generic Panel

class Panel:

	def get_summary(self):
		return "<default summary>"

	def get_description(self):
		return "<default description>"

	def check(self):
		return TRUE

	def show(self, dialog):
		return GtkVBox()
	
	def hide(self, dialog):
		pass

# ------------------------------------------------------------------------------
# Welcome panel

class WelcomePanel(Panel):

	def get_summary(self):
		return _("Website Synchronization Wizard")

	def get_description(self):
		return _("This Wizard will help you create a new group and synchronization, "	\
				 "that DrakSync will use to maintain your remote web site synchronized " \
				 "with your local copy.") + "\n\n" + \
			   _("In order to accomplish that, please answer the questions on the following screens.")

# ------------------------------------------------------------------------------
# Server Information Panel

class ServerInfoPanel(Panel):

	def get_summary(self):
		return _("Server and Authentification")

	def get_description(self):
		return _("Please enter details about the remote server hosting your web site.") + "\n\n" + \
			   _("Your password will not appear on the screen.") + " " + \
			   _("For security reasons, we don't remember your password by default but if you wish it, " \
				 "please check the corresponding box below.")

	def check(self):
		check_ok =	(self.hostname.get_text() != FTPInfo.DEFAULT_HOSTNAME) and \
					(self.username.get_text() != FTPInfo.DEFAULT_USERNAME) and \
					(self.password.get_text() != FTPInfo.DEFAULT_PASSWORD)
		if not check_ok:
			GuiPopupQuestion(PI_NAME + " - " + _("Missing entries"),
							 _("Please make sure to fill in all the entries."), (None, None),
							 ((_("OK"), None, None, GRAB_DEFAULT),)).show()
		return check_ok
	
	def show(self, dialog):
		box = GtkVBox()
		table = make_table(box, 2, 4)
		self.hostname = make_table_entry(table, 0, _("Host name"), dialog.data.hostname)
		self.username = make_table_entry(table, 1, _("User name"), dialog.data.username)
		self.password = make_table_entry(table, 2, _("Password"), dialog.data.password)
		self.password.set_visibility(FALSE)
		self.remember_password = dialog.data.remember_password
		make_table_checkbox(table, 3, _("Remember password"), self.remember_password, self.__cb_remember_password)
		return box

	def hide(self, dialog):
		dialog.data.hostname = self.hostname.get_text()
		dialog.data.username = self.username.get_text()
		dialog.data.password = self.password.get_text()
		dialog.data.remember_password = self.remember_password

	def __cb_remember_password(self, *args):
		button = args[0]
		self.remember_password = button.active

# ------------------------------------------------------------------------------
# Directories Panel

class DirectoriesPanel(Panel):

	def get_summary(self):
		return _("Local and Remote Directories")

	def get_description(self):
		return _("Please specify where the files of your web site are stored; both locally and remotely.") + "\n\n" + \
			   _("Directories shall be either absolute (e.g. /home/gb/website/) or relative to your login directory (e.g. ~/).")

	def show(self, dialog):
		box = GtkVBox()
		table = make_table(box, 2, 2)
		local_dir_selector = GtkButton(_("..."))
		local_dir_selector.connect("clicked", self.__cb_select_dir)
		self.local_dir = make_table_entry(table, 0, _("Local directory"), dialog.data.local_dir, local_dir_selector)
		self.remote_dir = make_table_entry(table, 1, _("Remote directory"), dialog.data.remote_dir)
		return box

	def hide(self, dialog):
		dialog.data.local_dir = self.local_dir.get_text()
		dialog.data.remote_dir = self.remote_dir.get_text()

	def check(self):
		if not os.path.isdir(self.local_dir.get_text()):
			GuiPopupQuestion(_("%s - Directory not found") % PI_NAME,
							 _("The specified local directory does not exist."),
							 (None, None), ((_("OK"), None, None),)).show()
			return FALSE
		
		return TRUE

	def __cb_select_dir(self, *args):
		GuiPopupDirectorySelection(PI_NAME + " - " + _("Please select local directory"),
								   self.local_dir.get_text(), ok=(self.__cb_selection_ok, None)).show()

	def __cb_selection_ok(self, path):
		self.local_dir.set_text(path)

	def __cb_selection_cancel(self, *args):
		self.file_selector.hide()

# ------------------------------------------------------------------------------
# Finish Panel

class FinishPanel(Panel):

	def get_summary(self):
		return _("Configuration Successful!")

	def get_description(self):
		return _("Thank you, enough information has now been gathered and a new synchronization " \
				 "will be created under the \"My Websites\" group.") + "\n\n" + \
			   _("Please click Finish to start synchronizing your web site.")

# ------------------------------------------------------------------------------
# Generic Wizard Dialog Box

WIZARD_DEFAULT_WIDTH	= 540
WIZARD_DEFAULT_HEIGHT	= 460

class WizardDialog(GtkDialog):

	class ui_btn_pressed:
		
		def __init__(self, dialog, callback):
			self.dialog = dialog
			self.callback = callback
			
		def __call__(self, button):
			apply(self.callback, ())
	
	def __init__(self, title, panels, data):
		WizardDialog.__bases__[0].__init__(self)
		_gtk.gtk_object_set_data(self._o, "userclass", self)
		self.set_position(WIN_POS_CENTER)
		self.set_title(title)
		self.set_default_size(WIZARD_DEFAULT_WIDTH, WIZARD_DEFAULT_HEIGHT)
		self.data = data
		self.panels = []
		self.panels_map = {}
		panel_id = 0
		for (key, panel) in panels:
			self.panels.append(panel)
			self.panels_map[key] = panel_id
			panel_id = panel_id + 1
		self.current_panel = 0

		self.action_area.set_homogeneous(FALSE)
		self.action_area.set_spacing(5)

		# Next button
		button = GtkButton(WIZ_BUTTON_NEXT)
		button.set_flags(CAN_DEFAULT)
		button.connect("clicked", self.ui_btn_pressed(self, self.__cb_next))
		self.action_area.pack_end(button, FALSE, TRUE, 0)
		self.next_button = button

		# Finish button
		button = GtkButton(WIZ_BUTTON_FINISH)
		button.set_flags(CAN_DEFAULT)
		button.connect("clicked", self.ui_btn_pressed(self, self.__cb_finish))
		self.action_area.pack_end(button, FALSE, TRUE, 0)
		self.finish_button = button

		# Back button
		button = GtkButton(WIZ_BUTTON_BACK)
		button.set_flags(CAN_DEFAULT)
		button.connect("clicked", self.ui_btn_pressed(self, self.__cb_back))
		self.action_area.pack_end(button, FALSE, TRUE, 0)
		self.back_button = button

		# Cancel button (leftmost button)
		button = GtkButton(WIZ_BUTTON_CANCEL)
		button.set_flags(CAN_DEFAULT)
		button.connect("clicked", self.ui_btn_pressed(self, self.__cb_cancel_confirm))
		self.action_area.pack_start(button, FALSE, TRUE, 0)
		self.cancel_button = button

		# Empty button set at start (filled in by __show_buttons)
		self.buttons = [ ]

		# Configured the website synchronization ?
		self.done = FALSE

		# Step summary
		self.summary_background = create_pixmap_from_xpm(self, self.get_style().bg[STATE_NORMAL], PI_ICONS_DIR + "/wiz_banner.xpm")
		self.summary_area = GtkDrawingArea()
		self.summary_area.size(WIZARD_DEFAULT_WIDTH, 100)
		self.summary_area.show()
		self.summary_area.signal_connect("expose_event", self.__expose_summary)
		self.vbox.pack_start(self.summary_area, FALSE, FALSE, 0)
		self.summary = GtkLabel()

		# Step description
		self.description = GtkLabel()
		self.description.set_line_wrap(TRUE)
		self.vbox.add(self.description)

	def __expose_summary(self, *args):
		panel = self.panels[self.current_panel]
		style = self.summary_area.get_style()
		black_gc = style.black_gc
		white_gc = style.white_gc
		big_font = load_fontset("-adobe-times-bold-r-normal--17-*-100-100-p-*-iso8859-*,*-r-*")
		self.summary_area.draw_pixmap(white_gc, self.summary_background[0], 0, 0, 0, 0, WIZARD_DEFAULT_WIDTH, 100)
		self.summary_area.draw_string(big_font, white_gc, 30, 60, panel.get_summary())

	def hide_panel(self):
		self.summary_area.hide()
		panel = self.panels[self.current_panel]
		panel.hide(self)
		self.core_box.hide_all()
		self.vbox.remove(self.core_box)

	def show_panel(self):
		panel = self.panels[self.current_panel]
		self.summary_area.show()
		self.description.set_text(panel.get_description())
		self.description.show()
		self.core_box = panel.show(self)
		self.core_box.set_border_width(16)
		self.core_box.show_all()
		self.vbox.add(self.core_box)
		for button in self.buttons:
			button.hide()
		self.__update_buttons()
		for button in self.buttons:
			button.show()

	def goto_panel(self, key):
		self.hide_panel()
		if self.panels_map.has_key(key):
			self.current_panel = self.panels_map[key]
		self.show_panel()

	def show(self):
		self.show_panel()
		WizardDialog.__bases__[0].show(self)

	def __update_buttons(self):
		if self.current_panel == 0:
			self.back_button.set_sensitive(FALSE)
			self.buttons = [ self.back_button, self.next_button, self.cancel_button ]
		elif self.current_panel == len(self.panels) - 1:
			self.buttons = [ self.back_button, self.finish_button, self.cancel_button ]
			for button in self.buttons:
				button.set_sensitive(TRUE)
		else:
			self.buttons = [ self.back_button, self.next_button, self.cancel_button ]
			for button in self.buttons:
				button.set_sensitive(TRUE)

	def __check(self):
		panel = self.panels[self.current_panel]
		return panel.check()

	def __cb_back(self):
		if self.__check():
			self.hide_panel()
			self.current_panel = self.current_panel - 1
			self.show_panel()

	def __cb_next(self):
		if self.__check():
			self.hide_panel()
			self.current_panel = self.current_panel + 1
			self.show_panel()

	def __cb_cancel(self):
		self.done = TRUE
		self.data = None

	def __cb_cancel_confirm(self):
		GuiPopupQuestion(_(PI_WIZARD) + " - " + _("Confirm quit"),
						 _("Are you sure you want to quit the Website Synchronization Wizard ? Any change will be lost."), (None, None),
						 ((_("Yes"), self.__cb_cancel, None), (_("No"), None, None, GRAB_DEFAULT))).show()

	def __cb_done(self):
		self.done = TRUE
		self.destroy()

	def __cb_connect_synchronize(self, network):
		network.connect()
		if network.isActive():
			self.__synchronize()
			network.disconnect()
		
		else:
			GuiPopupQuestion(_(PI_WIZARD) + " - " + _("Network failure"),
							 _("Sorry, I could not start the network. Please check your configuration and try again later."),
							 (None, None), ((_("OK"), self.__cb_done, None),)).show()

	def __synchronize(self):
		if DEBUG:
			print "Wizard::synchronize()"

		try:
			FTPTransfer(ST_LOCAL_IS_NEWER,			# Local repository is always newer
						TRUE,						# Delete files on remote site if they don't exist locally
						self.data,					# FTPInfo
						TRUE)						# Raise Error() if synchronization failed
			
		except (Error.FTPError, Error.ConnectionClosed):
			# We can't do anything more, that's an FTP error
			self.__cb_done()
		except (Error.UnknownHost, Error.LoginFailed):
			self.goto_panel("server")
		except Error.DirectoryNotFound:
			self.goto_panel("directories")
		else:
			GuiPopupQuestion(_(PI_WIZARD),
							 _("Your website has been successfully synchronized.") + "\n\n" +
							 _("Please do not forget to save your new configuration."),
							 (None, None), ((_("OK"), self.__cb_done, None),)).show()

	def __cb_finish(self):
		network = Network()
		
		#- a: if the network is setup and active
		if network.isActive():
			# synchronize right away
			self.__synchronize()
		
		#- b: if the network is setup but inactive
		elif network.isSetup():
			# if the user wishes it, synchronize
			GuiPopupQuestion(_(PI_WIZARD) + " - " + _("Network down"),
							 _("Your network appears to be down. Do you want me to activate it and start synchronizing your web site now ?"),
							 (None, None), ((_("Yes"), self.__cb_connect_synchronize, (network,), GRAB_DEFAULT), (_("No"), self.__cb_done, None))).show()
			
		#- c: if the network is not setup
		else:
			# if the user agrees, set it up and synchronize now
			# FIXME: unimplemented yet
			pass

# ------------------------------------------------------------------------------
# WebSync Wizard

class Wizard:

	def __init__(self):
		self.dialog = WizardDialog(PI_NAME + " - " + _("Website Synchronization Wizard"),
								   [ ("welcome", WelcomePanel()),
									 ("server", ServerInfoPanel()),
									 ("directories", DirectoriesPanel()),
									 ("finish", FinishPanel()) ],
								   FTPInfo())
		
	def run(self):
		self.dialog.done = FALSE
		self.dialog.show()
		while not self.dialog.done:
			while events_pending():
				mainiteration()
		self.dialog.hide()
		return self.dialog.data

if __name__ == "__main__":
	wiz = Wizard()
	webinfo = wiz.run()
	print webinfo
